import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)

/**
 * this is the board for the game Gomoku (connect 5)
 * 
 * @RUMMAKER / Louie Shen
 * @version beta
 */
public class Board extends World
{
    public final int BLANK_;
    public final int BLACK_;
    public final int WHITE_;
    public int[][] board_;
    public int x_;
    public int y_;
    public int player_;
    public Ring r;
    public Board()
    {    
        super(19, 19, 30);
        BLANK_ = 0;// no pieces on point
        BLACK_ = 1;// black piece
        WHITE_ = 2;// white piece
        board_ = new int[19][19];// a 19 by 19 Go/Gomoku board
        x_ = 0;
        y_ = 0;
        player_ = BLACK_;//black starts first
        r = new Ring();
        addObject(r, 0, 0);
        setPaintOrder(Ring.class);
    }
    public void act()
    {
        if(Greenfoot.getMouseInfo() != null)
        {
            if(Greenfoot.mouseClicked(this))
            {
                x_ = Greenfoot.getMouseInfo().getX();
                y_ = Greenfoot.getMouseInfo().getY();
                if(player_ == BLACK_)
                {
                    addBlack();
                }
                else
                {
                    addWhite();
                }
            }
            scanForWin();
        }
    }
    public void addBlack()
    {
        if(board_[x_][y_] == BLANK_)
        {
            board_[x_][y_] = BLACK_;
            Black b = new Black();
            addObject(b, x_, y_);
            player_ = WHITE_;// switch turns
        }
    }
    public void addWhite()
    {
        if(board_[x_][y_] == BLANK_)
        {
            board_[x_][y_] = WHITE_;
            White w = new White();
            addObject(w, x_, y_);
            player_ = BLACK_;// switch turns
        }
    }
    
    public boolean scanVertically(int win, int player_)//how many stones to scan for, what color stone to find (looks for the opposite color because it scanns after the player switches)
    {
        int y = y_;
        int stoneCount = 1;
        
        while(y > 0 && board_[x_][y-1] != player_)//doing scan includeing blank for the AI to see if its possible to connect five
        {
            y--;
            stoneCount++;
        }
        y = y_;
        while(y < 18 && board_[x_][y+1] != player_)
        {
            y++;
            stoneCount++;
        }
        if(stoneCount >= 5)//using 5 instead of win because this step is to help the AI decide if its possible to win
        {
            y = y_;
            stoneCount = 1;
        
            while(y > 0 && board_[x_][y-1] != player_ && board_[x_][y-1] != BLANK_)
            {
                y--;
                stoneCount++;
            }
            y = y_;
            while(y < 18 && board_[x_][y+1] != player_ && board_[x_][y+1] != BLANK_)
            {
                y++;
                stoneCount++;
            }
            return stoneCount >= win;
        }
        else
        {
            return false;
        }
    }
    
    public boolean scanHorizontally(int win, int player_)
    {
        int x = x_;
        int stoneCount = 1;
        
        while(x > 0 && board_[x-1][y_] != player_)
        {
            x--;
            stoneCount++;
        }
        x = x_;
        while(x < 18 && board_[x+1][y_] != player_)
        {
            x++;
            stoneCount++;
        }
        if(stoneCount >= 5)
        {
            x = x_;
            stoneCount = 1;
        
            while(x > 0 && board_[x-1][y_] != player_ && board_[x-1][y_] != BLANK_)
            {
                x--;
                stoneCount++;
            }
            x = x_;
            while(x < 18 && board_[x+1][y_] != player_ && board_[x+1][y_] != BLANK_)
            {
                x++;
                stoneCount++;
            }
            return stoneCount >= win;
        }
        else
        {
            return false;
        }
    }
    
    public boolean scanRLD(int win, int player_)//right-left-down
    {
        int y = y_;
        int x = x_;
        int stoneCount = 1;
        
        while(x < 18 && y > 0 && board_[x+1][y-1] != player_)
        {
            x++;
            y--;
            stoneCount++;
        }
        y = y_;
        x = x_;
        while(x > 0 && y < 18 && board_[x-1][y+1] != player_)
        {
            x--;
            y++;
            stoneCount++;
        }
        if(stoneCount >= 5)
        {
            y = y_;
            x = x_;
            stoneCount = 1;
        
            while(x < 18 && y > 0 && board_[x+1][y-1] != player_ && board_[x+1][y-1] != BLANK_)
            {
                x++;
                y--;
                stoneCount++;
            }
            y = y_;
            x = x_;
            while(x > 0 && y < 18 && board_[x-1][y+1] != player_ && board_[x-1][y+1] != BLANK_)
            {
                x--;
                y++;
                stoneCount++;
            }
            return stoneCount >= win;
        }
        else
        {
            return false;
        }
    }
    public boolean scanLRD(int win, int player_)//left-right-down
    {
        int y = y_;
        int x = x_;
        int stoneCount = 1;
        
        while(x > 0 && y > 0 && board_[x-1][y-1] != player_)
        {
            x--;
            y--;
            stoneCount++;
        }
        y = y_;
        x = x_;
        while(x < 18 && y < 18 && board_[x+1][y+1] != player_)
        {
            x++;
            y++;
            stoneCount++;
        }
        if(stoneCount >= 5)
        {
            y = y_;
            x = x_;
            stoneCount = 1;
        
            while(x > 0 && y > 0 && board_[x-1][y-1] != player_ && board_[x-1][y-1] != BLANK_)
            {
                x--;
                y--;
                stoneCount++;
            }
            y = y_;
            x = x_;
            while(x < 18 && y < 18 && board_[x+1][y+1] != player_ && board_[x+1][y+1] != BLANK_)
            {
                x++;
                y++;
                stoneCount++;
            }
            return stoneCount >= win;
        }
        else
        {
            return false;
        }
    }
    public boolean scanAll(int win, int player_)
    {
        if(scanVertically(win, player_) || scanHorizontally(win, player_) || scanLRD(win, player_) || scanRLD(win, player_))
        {
            return true;
        }
        else
        {
            return false;
        }
    }
    public int getX()
    {
        return x_;
    }
    public int getY()
    {
        return y_;
    }
    public int getBoard(int x, int y)
    {
        return board_[x_][y_];
    }
    public int getPlayer()
    {
        return player_;
    }
    public void scanForWin()
    {
        r.setLocation(x_, y_);
        if(scanAll(5, player_))
        {
            Message m = new Message();
            if(player_ == BLACK_)
            {
                GreenfootImage i = new GreenfootImage("whiteWin.png");
                i.setTransparency(120);
                m.setImage(i);
            }
            else
            {
                GreenfootImage i = new GreenfootImage("blackWin.png");
                i.setTransparency(120);
                m.setImage(i);
            }
            addObject(m, getWidth()/2, getHeight()/2);
            Greenfoot.stop();
        }
    }
}
